#include <vector>
#include <cassert>
#include <cstring>
#include <iostream>
#include <random>
#include <algorithm>
#include <cstdint>
#include <map>
#include <queue>
typedef long long ll;
using namespace std;

/* The state must be seeded so that it is not all zero */
class xorshift128plus{
  uint64_t state[2];
  public:
  xorshift128plus(uint64_t _state[2]){
    state[0] = _state[0];
    state[1] = _state[1];
  }
  xorshift128plus(initializer_list<uint64_t> il){
    copy(il.begin(), il.end(), state);
  }
  uint64_t operator()(){
    uint64_t t = state[0];
    uint64_t const s = state[1];
    state[0] = s;
    t ^= t << 23;		// a
    t ^= t >> 17;		// b
    t ^= s ^ (s >> 26);	// c
    state[1] = t;
    return t + s;
  }
  uint64_t max(){
    return -1;
  }
  uint64_t min(){
    return 0;
  }
};

vector<vector<short> > bit_vecs;
vector<char> dp;
int b, m;
int bucket_cnt;
const char INF = 64;
const char M_LIM = 10;
uint64_t def_seed = 64947;
//b=16, m=8
//33673 - 3.765

//b=20, m=10
//64947 - 3.765
//b=4, m=2
//948706 - 4.13
//b=2, m=1
//155385 - 5.01
void gen_vecs(int m, uint64_t seed = def_seed){
  xorshift128plus gen({0xbf58476d1ce4e5b9*seed, 0x2545f4914f6cdd1d*seed});
  for(int i=0; i<10; ++i)
    gen();
  bit_vecs[m].clear();
  for(int i=0; i<b; ++i){
    uint64_t r = gen();
    r ^= r>>32;
    r ^= r>>16;
    bit_vecs[m].push_back(r&((1<<m)-1));
  }
}

void init(int bb, int mm){
  b = bb;
  m = mm;
  bucket_cnt = (m+M_LIM-1)/M_LIM;
  bit_vecs.resize(M_LIM+1);
  for(int cm=1; cm<=M_LIM; ++cm){
    gen_vecs(cm);
  }
}

void init_encode(int b, int m){
  init(b, m);
  dp.resize((b+1)*(1<<M_LIM));
}

bool small_encode(vector<bool> &mem, int val, int m, int min_extra_ones = 0){
  int b = mem.size();
  if(!(rand()&((1<<0)-1))){
    cerr<<"b="<<b<<", m="<<m<<'\n';
  }
  memset(dp.data(), INF, 1<<m);
  dp[0] = 0;
  for(int i=0; i<b; ++i){
    if(!mem[i]){
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + j] = dp[i*(1<<m) + j];
      }
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])] = min((int)dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])], dp[i*(1<<m)+j]+1);
      }
    } else {
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])] = dp[i*(1<<m) + j];
      }
    }
  }
  if(dp[b*(1<<m)+val] >= INF)
    return false;
  int cnt = dp[b*(1<<m)+val];
  for(int i=b-1; i>=0; --i){
    if(!mem[i]){
      if(dp[i*(1<<m) + (val^bit_vecs[m][i])] == cnt-1){
        --cnt;
        --min_extra_ones;
        val ^= bit_vecs[m][i];
        mem[i] = true;
      } else if(cnt < min_extra_ones && dp[i*(1<<m) + (val^bit_vecs[m][i])] <= min_extra_ones-1){
        cnt = dp[i*(1<<m) + (val^bit_vecs[m][i])];
        --min_extra_ones;
        val ^= bit_vecs[m][i];
        mem[i] = true;
      }
    } else {
      val ^= bit_vecs[m][i];
    }
  }
  if(min_extra_ones > 0)
    return false;
  return true;
}

const int min_zero_cnt[] = {0, 1, 1, 1, 1, 4, 4, 4, 4, 4, 4};

bool encode(vector<bool> &mem, vector<bool> message){
  xorshift128plus gen({0xbf58486d1ce4dd1d, 0x2545f4914f6ce5b9});
  for(int i=0; i<100; ++i)
    gen();
  int mem_l = 0;
  int message_l = 0;
  while((int)message.size()-message_l > M_LIM){
    if(mem_l == (int)mem.size()){
      for(int i=message_l; i<(int)message.size(); ++i){
        if(message[i]) return false;
      }
      return true;
    }
    double r = ((double)mem.size()-mem_l)/((int)message.size()-message_l);
    const double mult = 0.93;
    int mem_len = r*M_LIM;
    if(mem_len > r*mult*M_LIM){
      double tar = r*mult*M_LIM;
      mem_len = tar;
      tar -= mem_len;
      if((double)(gen()&((1LL<60)-1))/(1LL<<60) < tar)
        ++mem_len;
    }
    mem_len = max(mem_len, 1);
    mem_len = min(mem_len, (int)mem.size()-mem_l);
    int mem_r = mem_l + mem_len;
    int message_len = M_LIM;
    while(message_len){
      int message_r = message_l+message_len;
      vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
      vector<bool> message_seg(message.begin()+message_l, message.begin()+message_r);
      int val = 0;
      for(int i=0; i<message_len; ++i){
        val |= (int)message_seg[i]<<i;
      }
      int min_extra_ones = 0;
      if(message_len < M_LIM){
        int pre_zero_cnt = 0;
        for(bool cur : mem_seg){
          if(!cur) ++pre_zero_cnt;
        }
        min_extra_ones = max(0, pre_zero_cnt-min_zero_cnt[message_len+1]+1);
      }
      bool suc = small_encode(mem_seg, val, message_len, min_extra_ones);
      int zero_cnt = 0;
      for(bool cur : mem_seg){
        if(!cur) ++zero_cnt;
      }
      if(suc && zero_cnt >= min_zero_cnt[message_len]){
        copy(mem_seg.begin(), mem_seg.end(), mem.begin()+mem_l);
        break;
      }
      --message_len;
    }
    if(!message_len){
      for(int i=mem_l; i<mem_r; ++i){
        mem[i] = true;
      }
    }
    mem_l += mem_len;
    message_l += message_len;
  }
  int mem_r = mem.size();
  int message_r = message.size();
  vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
  vector<bool> message_seg(message.begin()+message_l, message.begin()+message_r);
  int val = 0;
  for(int i=0; i<(int)message_seg.size(); ++i){
    val |= (int)message_seg[i]<<i;
  }
  bool suc = small_encode(mem_seg, val, message_seg.size());
  if(!suc)
    return false;
  copy(mem_seg.begin(), mem_seg.end(), mem.begin()+mem_l);
  cerr<<"Encoded\n\n";
  return true;
}

void init_decode(int b, int m){
  init(b, m);
}

int small_decode(vector<bool> mem, int m){
  int res = 0;
  int b = mem.size();
  for(int i=0; i<b; ++i){
    if(mem[i]){
      res ^= bit_vecs[m][i];
    }
  }
  return res;
}

vector<bool> decode(vector<bool> mem){
  vector<bool> message(m);
  xorshift128plus gen({0xbf58486d1ce4dd1d, 0x2545f4914f6ce5b9});
  for(int i=0; i<100; ++i)
    gen();
  int mem_l = 0;
  int message_l = 0;
  while(m-message_l > M_LIM){
    if(mem_l == (int)mem.size()){
      return message;
    }
    double r = ((double)mem.size()-mem_l)/(m-message_l);
    const double mult = 0.93;
    int mem_len = r*M_LIM;
    if(mem_len > r*mult*M_LIM){
      double tar = r*mult*M_LIM;
      mem_len = tar;
      tar -= mem_len;
      if((double)(gen()&((1LL<60)-1))/(1LL<<60) < tar)
        ++mem_len;
    }
    mem_len = max(mem_len, 1);
    mem_len = min(mem_len, (int)mem.size()-mem_l);
    int mem_r = mem_l + mem_len;
    vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
    int message_len = M_LIM;
    int zero_cnt = 0;
    for(bool cur : mem_seg){
      if(!cur) ++zero_cnt;
    }
    while(message_len){
      if(zero_cnt >= min_zero_cnt[message_len])
        break;
      --message_len;
    }
    if(message_len){
      int val = small_decode(mem_seg, message_len);
      vector<bool> message_seg;
      for(int i=0; i<message_len; ++i){
        message_seg.push_back(val & (1<<i));
      }
      copy(message_seg.begin(), message_seg.end(), message.begin()+message_l);
    }
    mem_l += mem_len;
    message_l += message_len;
  }
  int mem_r = mem.size();
  int message_r = m;
  if(message_l != message_r){
    vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
    int val = small_decode(mem_seg, message_r-message_l);
    vector<bool> message_seg;
    for(int i=0; i<message_r-message_l; ++i){
      message_seg.push_back(val & (1<<i));
    }
    copy(message_seg.begin(), message_seg.end(), message.begin()+message_l);
  }
  return message;
}

int cycle_len(int b, int m, int seed){
  static xorshift128plus gen({0xbf58486d1ce4dd1d, 0x2545f4914f6ce5b9});
  gen_vecs(m, seed);
  vector<bool> mem(b);
  for(int i=0; ; ++i){
    uint64_t r = gen();
    r ^= r>>32;
    r ^= r>>16;
    if(!small_encode(mem, r &((1<<m)-1), m))
      return i;
  }
}

//deg-1, 2-pow
vector<vector<double> > st_beg = {
{ 0, 1.00103, 2.41561, 5.02992, 10.1657, 20.3648, 40.7975, 81.555, 162.693, 323.554, 651.377, 1318.35, 2664.19, 5140.35, 10108.5, 20208.1, 39115.5, 80616.1, 140373, 329234, 823957, 996495, 7.69743e+06, 1.73324e+07},
{ 0, 0.816632, 1.60336, 2.55582, 3.80981, 5.52545, 7.91371, 11.2653, 15.9539, 22.5605, 31.8654, 45.2484, 63.6181, 89.9938, 128.633, 184.492, 258.824, 363.72, 491.651, 662.806, 1034.59, 1406.43, 3274.89, 4056.51},
{ 0, 0.765222, 1.42287, 2.1132, 2.89891, 3.83496, 4.97304, 6.37615, 8.13063, 10.3147, 13.0741, 16.4646, 20.694, 26.3224, 33.3271, 41.5139, 52.3437, 66.5906, 83.0852, 102.793, 124.532, 169.549, 201.056, 231.753},
{ 0, 0.740608, 1.345, 1.93585, 2.56251, 3.2549, 4.03724, 4.9375, 5.98605, 7.21291, 8.64331, 10.3359, 12.4141, 14.7924, 17.7176, 21.3547, 25.2358, 30.3876, 35.3733, 43.4482, 50.159, 56.6875, 81.5971, 85.3028},
{ 0, 0.726377, 1.30075, 1.8407, 2.38944, 2.96748, 3.59614, 4.29021, 5.06154, 5.93036, 6.91623, 8.0185, 9.26595, 10.6552, 12.3251, 14.15, 16.1604, 18.9428, 21.69, 25.1009, 28.4924, 33.5399, 42.0144, 75.1659},
{ 0, 0.717238, 1.27291, 1.78202, 2.28343, 2.79759, 3.34029, 3.92184, 4.55222, 5.24486, 5.98551, 6.82129, 7.72385, 8.75737, 9.91392, 11.1792, 12.517, 13.8933, 15.3643, 16.6939, 18.6368, 21.2132, 27.6629, 31.7604},
{ 0, 0.710861, 1.25391, 1.7415, 2.2126, 2.68652, 3.17311, 3.68218, 4.22465, 4.8006, 5.42137, 6.07365, 6.79082, 7.56418, 8.40892, 9.37625, 10.5144, 11.5846, 12.7255, 14.0085, 15.4828, 16.8628, 19.4236, 21.6676},
{ 0, 0.705946, 1.23958, 1.71296, 2.16253, 2.60733, 3.05812, 3.52302, 4.01195, 4.5192, 5.06041, 5.64685, 6.26619, 6.94496, 7.68185, 8.46314, 9.32088, 10.319, 11.3543, 12.378, 13.5215, 14.5787, 17.0924, 19.2711},
{ 0, 0.702621, 1.22948, 1.69171, 2.12514, 2.54945, 2.97481, 3.40639, 3.85128, 4.31313, 4.79604, 5.31053, 5.83135, 6.40613, 6.98088, 7.62083, 8.3009, 8.99682, 9.63308, 10.1741, 10.8962, 11.5268, 12.506, 13.2816},
{ 0, 0.699723, 1.22067, 1.67378, 2.0954, 2.50267, 2.90729, 3.31523, 3.73033, 4.157, 4.59784, 5.05933, 5.53234, 6.04882, 6.56379, 7.07153, 7.62921, 8.20358, 8.81186, 9.47667, 10.2326, 11.0085, 12.2083, 12.9689},
{ 0, 0.697569, 1.21495, 1.66159, 2.0738, 2.46851, 2.85613, 3.24133, 3.6336, 4.03365, 4.44931, 4.87696, 5.31363, 5.78172, 6.27792, 6.77535, 7.37364, 7.92342, 8.55494, 9.27298, 10.1415, 10.6202, 11.1414, 12.4626},
{ 0, 0.695327, 1.20877, 1.64883, 2.0534, 2.43873, 2.81405, 3.18694, 3.56155, 3.94328, 4.3355, 4.73845, 5.14531, 5.57579, 6.04158, 6.5188, 7.00519, 7.50045, 7.94907, 8.56411, 9.20572, 10.0453, 10.6542, 11.4084},
{ 0, 0.694128, 1.20456, 1.64006, 2.03759, 2.41366, 2.77798, 3.1376, 3.49827, 3.86571, 4.23749, 4.62639, 5.01699, 5.41151, 5.82539, 6.24044, 6.65454, 7.16419, 7.61217, 8.0985, 8.66291, 9.13315, 9.43419, 10.0036},
{ 0, 0.692436, 1.19998, 1.63167, 2.02315, 2.3932, 2.74942, 3.10231, 3.45119, 3.79936, 4.15691, 4.52316, 4.89758, 5.26613, 5.64924, 6.03545, 6.43915, 6.84286, 7.25954, 7.74743, 8.2331, 8.71465, 9.1547, 9.97369},
{ 0, 0.691415, 1.19672, 1.62523, 2.01308, 2.37626, 2.72608, 3.06765, 3.40672, 3.74418, 4.08095, 4.41898, 4.77267, 5.12557, 5.50227, 5.85959, 6.23421, 6.65439, 7.0345, 7.47788, 8.13271, 8.73899, 9.24332, 9.72803},
{ 0, 0.69022, 1.19398, 1.6193, 2.00247, 2.36172, 2.70603, 3.03994, 3.36819, 3.69688, 4.03107, 4.36427, 4.7014, 5.03145, 5.39976, 5.79162, 6.18529, 6.5629, 6.95372, 7.30738, 7.75419, 7.99648, 8.86523, 9.65651},
{ 0, 0.689389, 1.19127, 1.61407, 1.99407, 2.3488, 2.6862, 3.01242, 3.33713, 3.65859, 3.97708, 4.30265, 4.62698, 4.96273, 5.29049, 5.60896, 5.94163, 6.27157, 6.6587, 7.04183, 7.46759, 8.06398, 8.3016, 8.57755},
{ 0, 0.688442, 1.18852, 1.60884, 1.98542, 2.33627, 2.66911, 2.99162, 3.30891, 3.62286, 3.93671, 4.24781, 4.56184, 4.89082, 5.22154, 5.55992, 5.91924, 6.25316, 6.5878, 6.91663, 7.27636, 7.5362, 8.07914, 8.18975},
{ 0, 0.687555, 1.18655, 1.60523, 1.97909, 2.32608, 2.65543, 2.97217, 3.28082, 3.58748, 3.89518, 4.19808, 4.50558, 4.81874, 5.11855, 5.44798, 5.74543, 6.08496, 6.38468, 6.70425, 7.05994, 7.2047, 8.33657, 8.57516},
{ 0, 0.686756, 1.18471, 1.60085, 1.97259, 2.31666, 2.64278, 2.95671, 3.26417, 3.56394, 3.86324, 4.16333, 4.47097, 4.76569, 5.06371, 5.35067, 5.67407, 6.01151, 6.31312, 6.71972, 7.09548, 7.44569, 7.83996, 8.14604},
{ 0, 0.686281, 1.18306, 1.59787, 1.96687, 2.30812, 2.63064, 2.94117, 3.24534, 3.54439, 3.83526, 4.11989, 4.41076, 4.69435, 4.98052, 5.2733, 5.56739, 5.84648, 6.24685, 6.63294, 6.86668, 7.08608, 7.3864, 7.53296},
{ 0, 0.685501, 1.18122, 1.59474, 1.96217, 2.30153, 2.62131, 2.9286, 3.22402, 3.51882, 3.80497, 4.08767, 4.36723, 4.64169, 4.92255, 5.20198, 5.46774, 5.7657, 6.01223, 6.3804, 6.64805, 6.87076, 7.38713, 7.91411},
{ 0, 0.685027, 1.1798, 1.59161, 1.95719, 2.29397, 2.60984, 2.91269, 3.20477, 3.49234, 3.77287, 4.05484, 4.33489, 4.60682, 4.87935, 5.12792, 5.42426, 5.73397, 6.03069, 6.29406, 6.58179, 6.908, 7.31169, 7.60649},
{ 0, 0.684648, 1.1786, 1.58964, 1.95338, 2.28779, 2.60114, 2.90064, 3.19056, 3.47583, 3.75399, 4.02574, 4.29949, 4.5734, 4.8573, 5.12797, 5.42988, 5.6905, 5.99612, 6.28587, 6.5466, 7.01375, 7.29502, 7.63556},
{ 0, 0.684506, 1.17778, 1.587, 1.94972, 2.28168, 2.59399, 2.89263, 3.18078, 3.46025, 3.73466, 4.00781, 4.27682, 4.5471, 4.82775, 5.09486, 5.35456, 5.64556, 5.88056, 6.22453, 6.45263, 6.90595, 7.10308, 7.86585},
{ 0, 0.684005, 1.17661, 1.58506, 1.94622, 2.27662, 2.58582, 2.8829, 3.16759, 3.44648, 3.72019, 3.99123, 4.25984, 4.5265, 4.79976, 5.05604, 5.34254, 5.64491, 5.88382, 6.15372, 6.38969, 6.64803, 7.02673, 7.31676},
{ 0, 0.683456, 1.17586, 1.58376, 1.94365, 2.2724, 2.58039, 2.8752, 3.15978, 3.4332, 3.69653, 3.95701, 4.21418, 4.47174, 4.72807, 4.97607, 5.21382, 5.45371, 5.68423, 5.96785, 6.21798, 6.54287, 6.72149, 6.92984},
{ 0, 0.683554, 1.17487, 1.58177, 1.94101, 2.26835, 2.57356, 2.86584, 3.14545, 3.4198, 3.68559, 3.94412, 4.19417, 4.43938, 4.68898, 4.95423, 5.20613, 5.49305, 5.74151, 5.96395, 6.24112, 6.50384, 6.66598, 6.72998},
{ 0, 0.682907, 1.17382, 1.57995, 1.93784, 2.26477, 2.5703, 2.85915, 3.13612, 3.40545, 3.66798, 3.92546, 4.18044, 4.43105, 4.69893, 4.93906, 5.1692, 5.40608, 5.62517, 5.87881, 6.31282, 6.56315, 6.95054, 7.02886}
};
vector<vector<double> > st_avg = {
{ 0.441637, 1.56481, 3.43336, 7.01853, 14.1061, 28.2385, 56.6058, 113.11, 224.772, 449.995, 909.768, 1842.11, 3628.1, 7085.79, 13837.9, 28201.5, 55469.2, 105475, 206427, 494113, 906617, 1.41813e+06, 1.12043e+07, 1.85774e+07},
{ 0.378981, 1.15749, 2.00598, 3.08036, 4.52164, 6.51628, 9.30547, 13.212, 18.6774, 26.4395, 37.441, 53.0218, 74.4426, 106.341, 153.31, 216.755, 303.529, 417.29, 557.41, 798.881, 1162.12, 1699.61, 3608.29, 6257.21},
{ 0.360028, 1.05735, 1.72282, 2.45088, 3.29917, 4.31846, 5.56802, 7.12198, 9.05887, 11.4807, 14.506, 18.2486, 23.0642, 29.3045, 36.8289, 45.5133, 58.3311, 74.3354, 91.9577, 109.849, 134.286, 179.359, 204.224, 333.134},
{ 0.350786, 1.01234, 1.60525, 2.20907, 2.8619, 3.59302, 4.42612, 5.38941, 6.51311, 7.82664, 9.37768, 11.2248, 13.4334, 16.0803, 19.3243, 22.9777, 27.6704, 32.447, 38.0505, 46.4474, 52.4798, 60.8884, 83.1808, 87.2984},
{ 0.345255, 0.986241, 1.54038, 2.08198, 2.64209, 3.24074, 3.89762, 4.62493, 5.43692, 6.35723, 7.39023, 8.55237, 9.86774, 11.3681, 13.132, 15.0369, 17.2225, 20.1709, 23.3166, 26.7332, 30.8083, 37.1634, 42.7085, 78.5206},
{ 0.341506, 0.969846, 1.50006, 2.00368, 2.50888, 3.03519, 3.59463, 4.19622, 4.85467, 5.56493, 6.34514, 7.21246, 8.17106, 9.26816, 10.4502, 11.7651, 13.1424, 14.5567, 16.0027, 17.439, 19.6437, 22.2298, 27.7921, 38.565},
{ 0.339083, 0.958597, 1.47228, 1.95049, 2.42173, 2.90077, 3.39632, 3.91883, 4.47733, 5.07263, 5.70807, 6.38768, 7.11742, 7.94408, 8.81189, 9.85842, 10.9491, 12.1032, 13.2086, 14.5416, 15.9676, 17.8533, 19.7461, 22.1218},
{ 0.337145, 0.950091, 1.45223, 1.91288, 2.3595, 2.80596, 3.26266, 3.7374, 4.23487, 4.75499, 5.31872, 5.91843, 6.5637, 7.26426, 8.01304, 8.82709, 9.73945, 10.7311, 11.7443, 12.792, 13.9253, 14.8939, 17.1975, 19.4167},
{ 0.336072, 0.943937, 1.43753, 1.88505, 2.31308, 2.73685, 3.1654, 3.60138, 4.0554, 4.52452, 5.02345, 5.54186, 6.07968, 6.66186, 7.249, 7.89915, 8.64289, 9.30775, 9.92725, 10.4821, 11.1921, 11.8385, 12.628, 14.6187},
{ 0.334808, 0.938677, 1.42517, 1.86209, 2.27631, 2.68159, 3.08715, 3.498, 3.91808, 4.34977, 4.80177, 5.27091, 5.75742, 6.28734, 6.79001, 7.31434, 7.85858, 8.47659, 9.09211, 9.81358, 10.4195, 11.1879, 12.3143, 13.6626},
{ 0.333908, 0.93507, 1.41663, 1.84593, 2.24956, 2.64069, 3.02689, 3.41442, 3.80938, 4.21675, 4.63858, 5.07189, 5.51466, 5.99679, 6.50115, 7.0363, 7.61525, 8.16071, 8.78047, 9.63731, 10.3616, 10.6806, 11.2899, 12.5089},
{ 0.332998, 0.931386, 1.4077, 1.83, 2.225, 2.60538, 2.97907, 3.35231, 3.72969, 4.11762, 4.51127, 4.91768, 5.33634, 5.77648, 6.24497, 6.72358, 7.22128, 7.71233, 8.20323, 8.75027, 9.59324, 10.2634, 10.6886, 11.7408},
{ 0.332477, 0.928979, 1.40143, 1.81843, 2.20538, 2.57557, 2.93731, 3.29668, 3.66116, 4.03002, 4.40925, 4.79737, 5.18974, 5.59274, 6.00177, 6.42178, 6.85198, 7.37423, 7.77214, 8.3199, 8.78886, 9.16577, 9.44663, 10.2853},
{ 0.331907, 0.926099, 1.39539, 1.80766, 2.18843, 2.55187, 2.90598, 3.25658, 3.60503, 3.95828, 4.31818, 4.68611, 5.06025, 5.43365, 5.81958, 6.19876, 6.61901, 7.02224, 7.43372, 7.94741, 8.47907, 8.82357, 9.52046, 10.0617},
{ 0.331338, 0.924246, 1.391, 1.7996, 2.1755, 2.53194, 2.87791, 3.2178, 3.55691, 3.8947, 4.23176, 4.57188, 4.92996, 5.29598, 5.6638, 6.02787, 6.44153, 6.82261, 7.20422, 7.69254, 8.27404, 9.05913, 9.24739, 10.089},
{ 0.330853, 0.922259, 1.38683, 1.7915, 2.16264, 2.51512, 2.85418, 3.18539, 3.51408, 3.84425, 4.17866, 4.51494, 4.84753, 5.19098, 5.56422, 5.95676, 6.34838, 6.72749, 7.09195, 7.48004, 7.87374, 8.44577, 8.93532, 9.74477},
{ 0.33064, 0.920723, 1.38313, 1.78485, 2.1526, 2.49916, 2.83168, 3.15638, 3.47969, 3.79928, 4.12017, 4.447, 4.77192, 5.10577, 5.43609, 5.76644, 6.0779, 6.44664, 6.80655, 7.17855, 7.58406, 8.11442, 8.38733, 9.4264},
{ 0.330282, 0.919133, 1.37929, 1.77845, 2.1425, 2.48469, 2.81226, 3.13276, 3.44813, 3.76167, 4.07315, 4.38973, 4.70392, 5.0389, 5.36502, 5.70421, 6.05065, 6.40804, 6.72469, 7.0805, 7.401, 7.7817, 8.12821, 8.84977},
{ 0.329911, 0.917635, 1.37669, 1.77362, 2.13445, 2.47273, 2.79671, 3.10947, 3.41639, 3.72409, 4.03005, 4.33339, 4.64278, 4.95384, 5.25445, 5.58075, 5.88363, 6.22592, 6.5315, 6.84339, 7.12095, 7.47063, 8.3944, 10.4518},
{ 0.329557, 0.916499, 1.37398, 1.76813, 2.12675, 2.4624, 2.78241, 3.09278, 3.39699, 3.69593, 3.99578, 4.29771, 4.59968, 4.89982, 5.18369, 5.49634, 5.831, 6.1505, 6.49542, 6.84266, 7.21828, 7.53923, 7.86907, 8.64237},
{ 0.329306, 0.915694, 1.37146, 1.76411, 2.11976, 2.4522, 2.76883, 3.07669, 3.37821, 3.67321, 3.96147, 4.24837, 4.5367, 4.82563, 5.10161, 5.40232, 5.69001, 6.00731, 6.39357, 6.70982, 6.93564, 7.15944, 7.42076, 7.5897},
{ 0.329144, 0.914437, 1.36918, 1.76048, 2.11441, 2.44437, 2.758, 3.05976, 3.35482, 3.64631, 3.93008, 4.21167, 4.48499, 4.76715, 5.0486, 5.31626, 5.59922, 5.8614, 6.16132, 6.5122, 6.71881, 7.07146, 7.40355, 8.0145},
{ 0.32896, 0.913481, 1.36683, 1.75667, 2.10816, 2.43519, 2.74502, 3.04266, 3.33205, 3.61682, 3.89788, 4.17639, 4.45358, 4.73133, 4.98912, 5.25756, 5.56512, 5.85921, 6.15236, 6.40082, 6.71795, 7.02967, 7.34109, 7.67353},
{ 0.328767, 0.912777, 1.36566, 1.75371, 2.1034, 2.42803, 2.73525, 3.02976, 3.3162, 3.59931, 3.87428, 4.14705, 4.41609, 4.70123, 4.97268, 5.26412, 5.53159, 5.80422, 6.11871, 6.38712, 6.68409, 7.10452, 7.43772, 7.95687},
{ 0.328676, 0.912407, 1.36392, 1.75049, 2.0987, 2.4214, 2.72763, 3.02052, 3.30461, 3.58216, 3.85459, 4.12759, 4.39503, 4.66688, 4.94854, 5.20482, 5.48889, 5.74063, 5.98728, 6.30374, 6.68738, 6.98128, 7.33375, 7.9651},
{ 0.328437, 0.911483, 1.36265, 1.74806, 2.09459, 2.41526, 2.71854, 3.00922, 3.29159, 3.56768, 3.84002, 4.11067, 4.37859, 4.64325, 4.91915, 5.18797, 5.46807, 5.76303, 6.00603, 6.24394, 6.43535, 6.7629, 7.16216, 7.58713},
{ 0.328245, 0.911005, 1.36153, 1.74608, 2.09126, 2.40996, 2.7122, 3.00165, 3.28178, 3.55069, 3.81133, 4.07258, 4.32768, 4.58806, 4.83681, 5.08801, 5.32925, 5.55858, 5.82135, 6.08553, 6.3493, 6.5982, 6.80815, 7.09305},
{ 0.328413, 0.910666, 1.35999, 1.74398, 2.08799, 2.40497, 2.70368, 2.99039, 3.2677, 3.53757, 3.80041, 4.05761, 4.30247, 4.55336, 4.80376, 5.06864, 5.33341, 5.58918, 5.84097, 6.07334, 6.33917, 6.56899, 6.67988, 7.13571},
{ 0.328058, 0.909951, 1.35876, 1.74147, 2.08479, 2.40195, 2.69922, 2.98263, 3.2554, 3.52276, 3.783, 4.03963, 4.28892, 4.5493, 4.80547, 5.04359, 5.26921, 5.50445, 5.72252, 6.00212, 6.41707, 6.66598, 6.98476, 7.11321}
};
vector<vector<double> > st_prob = {
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08},
{ 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.0078125, 0.00390625, 0.00195312, 0.000976562, 0.000488281, 0.000244141, 0.00012207, 6.10352e-05, 3.05176e-05, 1.52588e-05, 7.62939e-06, 3.8147e-06, 1.90735e-06, 9.53674e-07, 4.76837e-07, 2.38419e-07, 1.19209e-07, 5.96046e-08, 5.96046e-08}
};

void init_student_t(){
  xorshift128plus gen({0xbf58486d1ce4dd1d, 0x2545f4914f6ce5b9});
  for(int i=0; i<100; ++i)
    gen();
  for(int deg=1; deg<30; ++deg){
    st_beg.push_back(vector<double>());
    st_avg.push_back(vector<double>());
    st_prob.push_back(vector<double>());
    vector<double> val_ord(1);
    student_t_distribution<double> distr(deg);
    for(int i=0; i<(1<<24)-1; ++i){
      val_ord.push_back(abs(distr(gen)));
    }
    sort(val_ord.begin(), val_ord.end());
    for(int i=1; i<=24; ++i){
      int l = (1<<24)-(1<<(24-i+1));
      int r = (1<<24)-(1<<(24-i));
      if(i==24){
        r = val_ord.size();
      }
      st_beg.back().push_back(val_ord[l]);
      double sum = 0;
      for(int j=l; j<r; ++j){
        sum += val_ord[j];
      }
      st_avg.back().push_back(sum/(r-l));
      st_prob.back().push_back(double(r-l)/val_ord.size()/2);
    }
  }
  cout<<'\n';
  for(auto cur_vec : st_beg){
    cout<<"{ ";
    bool f = true;
    for(auto cur : cur_vec){
      if(!f)
        cout<<", ";
      f = false;
      cout<<cur;
    }
    cout<<"},\n";
  }
  cout<<'\n';
  for(auto cur_vec : st_avg){
    cout<<"{ ";
    bool f = true;
    for(auto cur : cur_vec){
      if(!f)
        cout<<", ";
      f = false;
      cout<<cur;
    }
    cout<<"},\n";
  }
  cout<<'\n';
  for(auto cur_vec : st_prob){
    cout<<"{ ";
    bool f = true;
    for(auto cur : cur_vec){
      if(!f)
        cout<<", ";
      f = false;
      cout<<cur;
    }
    cout<<"},\n";
  }
}

double exp_imp(double orig_tar, int cnt, double sum, double sum_sq){
  assert(cnt != 1);
  double res = 0;
  double avg = sum /cnt;
  double sample_var = (sum_sq/cnt-avg*avg)/(cnt -1);
  double tar = orig_tar - avg;
  if(sample_var < 1e-6)
    return tar;
  tar *= sqrt(cnt/sample_var);
  int idx = min(cnt -2, (int)st_beg.size()-1);
  for(int i=0; i<(int)st_beg[idx].size(); ++i){
    if(st_beg[idx][i] >= tar)
      res += st_prob[idx][i]*(st_avg[idx][i]-tar);
  }
  res *= sqrt(sample_var/cnt);
  return res;
}

double bad_case_avg(int cnt, double sum, double sum_sq){
  assert(cnt != 1);
  double avg = sum /cnt;
  double sample_var = (sum_sq/cnt-avg*avg)/(cnt -1);
  if(sample_var < 1e-6)
    return 0;
  int idx = min(cnt -2, (int)st_beg.size()-1);
  return avg-st_beg[idx][11]*sqrt(sample_var/cnt);
}

int best_seed(int b, int m, int pull_lim){
  init_encode(b, m);
  vector<int> cnt;
  vector<double> sum;
  vector<double> sum_sq;
  priority_queue<pair<double, int> > avg;
  priority_queue<pair<double, int> > exp_imp_ord;
  int pull_cnt = 0;
  for(int seed=0; seed<pull_lim/8; ++seed){
    ++pull_cnt;
    double cur = cycle_len(b, m, seed);
    cnt.push_back(1);
    sum.push_back(cur);
    sum_sq.push_back(cur*cur);
    avg.emplace(cur, seed);
  }
  for(int repull = pull_lim/16; repull>0; repull /= 2){
    vector<int> add_back;
    for(int i=0; i<repull; ++i){
      int seed = avg.top().second;
      avg.pop();
      ++pull_cnt;
      double cur = cycle_len(b, m, seed);
      ++cnt[seed];
      sum[seed] += cur;
      sum_sq[seed] += cur*cur;
      add_back.push_back(seed);
    }
    for(int seed : add_back){
      avg.emplace(sum[seed]/cnt[seed], seed);
    }
  }
  while(cnt[avg.top().second] < 10){
    int seed = avg.top().second;
    avg.pop();
    ++pull_cnt;
    double cur = cycle_len(b, m, seed);
    ++cnt[seed];
    sum[seed] += cur;
    sum_sq[seed] += cur*cur;
    avg.emplace(sum[seed]/cnt[seed], seed);
  }
  avg = priority_queue<pair<double, int> >();
  for(int seed = 0; seed<(int)cnt.size(); ++seed){
    if(cnt[seed] > 1){
      avg.emplace(sum[seed]/cnt[seed], seed);
      exp_imp_ord.emplace(exp_imp(avg.top().first, cnt[seed], sum[seed], sum_sq[seed]), seed);
    }
  }
  while(pull_cnt < pull_lim){
    int seed = exp_imp_ord.top().second;
    exp_imp_ord.pop();
    if(seed == avg.top().second)
      continue;
    double cur_exp_imp = exp_imp(avg.top().first, cnt[seed], sum[seed], sum_sq[seed]);
    if(cur_exp_imp >= exp_imp_ord.top().first){
      int best_seed = avg.top().second;
      if(10*cnt[seed] < cnt[best_seed]){
        ++pull_cnt;
        double cur = cycle_len(b, m, seed);
        ++cnt[seed];
        sum[seed] += cur;
        sum_sq[seed] += cur*cur;
        avg.emplace(sum[seed]/cnt[seed], seed);
        if(avg.top().second == seed){
          exp_imp_ord.emplace(exp_imp(avg.top().first, cnt[best_seed], sum[best_seed], sum_sq[best_seed]), best_seed);
        } else {
          exp_imp_ord.emplace(exp_imp(avg.top().first, cnt[seed], sum[seed], sum_sq[seed]), seed);
        }
      } else {
        avg.pop();
        ++pull_cnt;
        double cur = cycle_len(b, m, best_seed);
        ++cnt[best_seed];
        sum[best_seed] += cur;
        sum_sq[best_seed] += cur*cur;
        avg.emplace(sum[best_seed]/cnt[best_seed], best_seed);
        if(avg.top().second == best_seed){
          exp_imp_ord.emplace(cur_exp_imp, seed);
        } else {
          exp_imp_ord.emplace(exp_imp(avg.top().first, cnt[best_seed], sum[best_seed], sum_sq[best_seed]), best_seed);
        }
      }
      int seed = avg.top().second;
      avg.pop();
      avg.emplace(sum[seed]/cnt[seed], seed);
      if(avg.top().second != seed || cnt[avg.top().second] < 10){
        exp_imp_ord.emplace(exp_imp(avg.top().first, cnt[seed], sum[seed], sum_sq[seed]), seed);
      }
      while(avg.top().second != seed || cnt[avg.top().second] < 10){
        seed = avg.top().second;
        avg.pop();
        if(sum[seed]/cnt[seed] >= avg.top().first && cnt[seed] < 10){
          ++pull_cnt;
          double cur = cycle_len(b, m, seed);
          ++cnt[seed];
          sum[seed] += cur;
          sum_sq[seed] += cur*cur;
        }
        avg.emplace(sum[seed]/cnt[seed], seed);
      }
      if(!(rand()&((1<<16)-1))){
        cout<<"Cur best "<<avg.top().second<<": "<<avg.top().first<<'\n';
        cout<<"cnt = "<<cnt[avg.top().second]<<'\n';
      }
    } else {
      exp_imp_ord.emplace(cur_exp_imp, seed);
    }
  }
  vector<pair<double, int> > fin_avg;
  for(int seed = 0; seed<(int)cnt.size(); ++seed){
    if(cnt[seed] > 20)
      fin_avg.emplace_back(bad_case_avg(cnt[seed], sum[seed], sum_sq[seed]), seed);
  }
  sort(fin_avg.rbegin(), fin_avg.rend());
  for(int i=0; i<30; ++i){
    cout<<fin_avg[i].second<<": "<<fin_avg[i].first<<", "<<cnt[fin_avg[i].second]<<'\n';
  }
  return fin_avg.front().second;
}

int main(){
  int type;
  cin>>type;
  int b, m;
  cin>>b>>m;
  if(type == 2){
    /*init_student_t();
      cout<<"init done"<<endl;
      */
    cout<<best_seed(b, m, 1000000)<<'\n';
    return 0;
  }
  if(type){
    init_decode(b, m);
    vector<bool> mem(b);
    string mem_str;
    string message_str(m, '0');
    while(true){
      bool cont;
      cin>>cont;
      if(!cont)
        break;
      cin>>mem_str;
      for(int i=0; i<b; ++i)
        mem[i] = mem_str[i]-'0';
      vector<bool> message = decode(mem);
      for(int i=0; i<m; ++i)
        message_str[i] = '0'+message[i];
      cout<<message_str<<endl;
    }
  } else {
    init_encode(b, m);
    vector<bool> mem(b);
    vector<bool> message(m);
    string mem_str;
    string message_str;
    while(true){
      bool cont;
      cin>>cont;
      if(!cont)
        break;
      cin>>mem_str>>message_str;
      for(int i=0; i<b; ++i)
        mem[i] = mem_str[i]-'0';
      for(int i=0; i<m; ++i)
        message[i] = message_str[i]-'0';
      bool suc = encode(mem, message);
      cout<<suc;
      if(suc){
        if(decode(mem) != message){
          cerr<<"Encode error"<<endl;
        }
        for(int i=0; i<b; ++i)
          mem_str[i] = '0'+mem[i];
        cout<<'\n'<<mem_str;
      }
      cout<<endl;
    }
  }
}
