#include <vector>
#include <cassert>
#include <cstring>
#include <iostream>
#include <random>
#include <algorithm>
#include <cstdint>
#include <map>
typedef long long ll;
using namespace std;

/* The state must be seeded so that it is not all zero */
uint64_t xorshift128plus(uint64_t state[2]) {
  uint64_t t = state[0];
  uint64_t const s = state[1];
  state[0] = s;
  t ^= t << 23;		// a
  t ^= t >> 17;		// b
  t ^= s ^ (s >> 26);	// c
  state[1] = t;
  return t + s;
}

vector<vector<short> > bit_vecs;
vector<char> dp;
int b, m;
int bucket_cnt;
const char INF = 64;
const char M_LIM = 10;

//b=20, m=10
//28140 - 3.72
//75862 - 3.73
//85458 - 3.74
//142816 - 3.757
//b=4, m=2
//948706 - 4.13
//b=2, m=1
//155385 - 5.01
void gen_vecs(int m, uint64_t seed = 1){
  uint64_t state[2] = {0xbf58476d1ce4e5b9*seed, 0x2545f4914f6cdd1d*seed};
  for(int i=0; i<10; ++i){
    xorshift128plus(state);
  }
  bit_vecs[m].clear();
  for(int i=0; i<b; ++i){
    uint64_t r = xorshift128plus(state);
    r ^= r>>32;
    r ^= r>>16;
    bit_vecs[m].push_back(r&((1<<m)-1));
  }
}

void init(int bb, int mm){
  b = bb;
  m = mm;
  bucket_cnt = (m+M_LIM-1)/M_LIM;
  bit_vecs.resize(M_LIM+1);
  for(int cm=1; cm<=M_LIM; ++cm){
    gen_vecs(cm);
  }
}

void init_encode(int b, int m){
  init(b, m);
  dp.resize((b+1)*(1<<M_LIM));
}

bool small_encode(vector<bool> &mem, int val, int m){
  int b = mem.size();
  memset(dp.data(), INF, 1<<m);
  dp[0] = 0;
  for(int i=0; i<b; ++i){
    if(!mem[i]){
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + j] = dp[i*(1<<m) + j];
      }
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])] = min((int)dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])], dp[i*(1<<m)+j]+1);
      }
    } else {
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])] = dp[i*(1<<m) + j];
      }
    }
  }
  if(dp[b*(1<<m)+val] >= INF)
    return false;
  int cnt = dp[b*(1<<m)+val];
  for(int i=b-1; i>=0; --i){
    if(!mem[i]){
      if(dp[i*(1<<m) + (val^bit_vecs[m][i])] == cnt-1){
        --cnt;
        val ^= bit_vecs[m][i];
        mem[i] = true;
      }
    } else {
      val ^= bit_vecs[m][i];
    }
  }
  return true;
}

bool encode(vector<bool> &mem, vector<bool> message){
  for(int i=0; i<bucket_cnt; ++i){
    int mem_l = i*b/bucket_cnt;
    int mem_r = (i+1)*b/bucket_cnt;
    int message_l = i*m/bucket_cnt;
    int message_r = (i+1)*m/bucket_cnt;
    vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
    vector<bool> message_seg(message.begin()+message_l, message.begin()+message_r);
    int val = 0;
    for(int i=0; i<(int)message_seg.size(); ++i){
      val |= (int)message_seg[i]<<i;
    }
    bool suc = small_encode(mem_seg, val, message_seg.size());
    if(!suc)
      return false;
    copy(mem_seg.begin(), mem_seg.end(), mem.begin()+mem_l);
  }
  return true;
}

void init_decode(int b, int m){
  init(b, m);
}

int small_decode(vector<bool> mem, int m){
  int res = 0;
  int b = mem.size();
  for(int i=0; i<b; ++i){
    if(mem[i]){
      res ^= bit_vecs[m][i];
    }
  }
  return res;
}

vector<bool> decode(vector<bool> mem){
  vector<bool> message(m);
  for(int i=0; i<bucket_cnt; ++i){
    int mem_l = i*b/bucket_cnt;
    int mem_r = (i+1)*b/bucket_cnt;
    //cerr<<"mem: "<<mem_l<<" "<<mem_r<<endl;
    int message_l = i*m/bucket_cnt;
    int message_r = (i+1)*m/bucket_cnt;
    //cerr<<"message: "<<message_l<<" "<<message_r<<endl;
    vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
    int val = small_decode(mem_seg, message_r-message_l);
    vector<bool> message_seg;
    for(int i=0; i<message_r-message_l; ++i){
      message_seg.push_back(val & (1<<i));
    }
    copy(message_seg.begin(), message_seg.end(), message.begin()+message_l);
  }
  return message;
}

int cycle_len(int b, int m, int seed, const vector<int> &messages){
  gen_vecs(m, seed);
  vector<bool> mem(b);
  for(int i=0; ; ++i){
    if(!small_encode(mem, messages[i], m))
      return i;
  }
}

int best_seed(int b, int m, int pull_lim){
  init_encode(b, m);
  uint64_t state[2] = {0xbf58486d1ce4dd1d, 0x2545f4914f6ce5b9};
  for(int i=0; i<100; ++i){
    xorshift128plus(state);
  }
  ll all_sum=0;
  ll all_cnt=0;
  int best = 1;
  ll best_sum = 0;
  ll best_cnt = 1;
  vector<double> fin_avg(2);
  vector<multimap<double, int> > k_avg;
  int pull_cnt = 0;
  for(int seed=2; pull_cnt<pull_lim ;++seed){
    if((xorshift128plus(state) & 111) == 0){
      vector<int> messages;
      for(int i=0; i<3*b+30; ++i){
        uint64_t r = xorshift128plus(state);
        r ^= r>>32;
        r ^= r>>16;
        messages.push_back(r &((1<<m)-1));
      }
      best_sum += cycle_len(b, m, best, messages);
      ++pull_cnt;
      ++best_cnt;
      if(best_cnt == (int)k_avg.size())
        k_avg.resize(k_avg.size()+1);
      fin_avg[best] = (double)best_sum/best_cnt;
      k_avg[best_cnt].emplace(fin_avg[best], best);
    }
    ll cur_sum = 0;
    ll cur_cnt = 0;
    double max_fin_avg;
    double cur_avg;
    do{
      vector<int> messages;
      for(int i=0; i<3*b+30; ++i){
        uint64_t r = xorshift128plus(state);
        r ^= r>>32;
        r ^= r>>16;
        messages.push_back(r &((1<<m)-1));
      }
      cur_sum += cycle_len(b, m, seed, messages);
      ++pull_cnt;
      ++cur_cnt;
      if(cur_cnt >= (int)k_avg.size())
        k_avg.resize(cur_cnt+1);
      cur_avg = (double)cur_sum/cur_cnt;
      auto it = k_avg[cur_cnt].lower_bound(cur_avg);
      max_fin_avg = 0;
      auto it2 = it;
      for(int i=0; it2 != k_avg[cur_cnt].end() &&  i<5; ++i, ++it2){
        max_fin_avg = max(max_fin_avg, fin_avg[it2->second]);
      }
      for(int i=0; it != k_avg[cur_cnt].begin() &&  i<5; ++i){
        --it;
        max_fin_avg = max(max_fin_avg, fin_avg[it->second]);
      }
      k_avg[cur_cnt].emplace(cur_avg, seed);
      if(cur_cnt == 1){
        all_sum += cur_sum;
        ++all_cnt;
        if(cur_sum*all_cnt < all_sum)
          break;
      }
    } while((cur_cnt < 1000  && cur_sum*best_cnt >= best_sum*cur_cnt) || (2*max_fin_avg - cur_avg > (double)best_sum/best_cnt && (cur_cnt < 1000 || (cur_cnt < best_cnt && cur_sum*best_cnt < best_sum*cur_cnt))));
    // 2 * diff can make it best and (low cnt or (cnt below best and not best))
    if(cur_sum*best_cnt > best_sum*cur_cnt){
      best = seed;
      best_sum = cur_sum;
      best_cnt = cur_cnt;
      cout<<"New best "<<best<<": "<<(double)best_sum/best_cnt<<'\n';
      cout<<best_cnt<<'\n';
    }
    fin_avg.push_back((double)cur_sum/cur_cnt);
  }
  return best;
}

int main(){
  int type;
  cin>>type;
  int b, m;
  cin>>b>>m;
  if(type == 2){
    best_seed(b, m, 1000000);
    return 0;
  }
  if(type){
    init_decode(b, m);
    vector<bool> mem(b);
    string mem_str;
    string message_str(m, '0');
    while(true){
      bool cont;
      cin>>cont;
      if(!cont)
        break;
      cin>>mem_str;
      for(int i=0; i<b; ++i)
        mem[i] = mem_str[i]-'0';
      vector<bool> message = decode(mem);
      for(int i=0; i<m; ++i)
        message_str[i] = '0'+message[i];
      cout<<message_str<<endl;
    }
  } else {
    init_encode(b, m);
    vector<bool> mem(b);
    vector<bool> message(m);
    string mem_str;
    string message_str;
    while(true){
      bool cont;
      cin>>cont;
      if(!cont)
        break;
      cin>>mem_str>>message_str;
      for(int i=0; i<b; ++i)
        mem[i] = mem_str[i]-'0';
      for(int i=0; i<m; ++i)
        message[i] = message_str[i]-'0';
      bool suc = encode(mem, message);
      cout<<suc;
      if(suc){
        if(decode(mem) != message){
          cerr<<"Encode error"<<endl;
        }
        for(int i=0; i<b; ++i)
          mem_str[i] = '0'+mem[i];
        cout<<'\n'<<mem_str;
      }
      cout<<endl;
    }
  }
}
