#include <vector>
#include <cassert>
#include <cstring>
#include <iostream>
typedef long long ll;
using namespace std;

vector<vector<int> > bit_vecs;
vector<char> dp;
int b, m;
int bucket_cnt;
const char INF = 64;
const char M_LIM = 11;

void init(int bb, int mm){
  b = bb;
  m = mm;
  bucket_cnt = (m+M_LIM-1)/M_LIM;
  bit_vecs.resize(M_LIM+1);
  for(int cm=1; cm<=M_LIM; ++cm){
    for(int i=0; i<b; ++i){
      bit_vecs[cm].push_back(rand() & ((1<<cm)-1));
    }
  }
}

void init_encode(int b, int m){
  init(b, m);
  dp.resize((b+1)*(1<<M_LIM));
}

bool small_encode(vector<bool> &mem, vector<bool> message){
  int val = 0;
  int b = mem.size();
  int m = message.size();
  memset(dp.data(), INF, 1<<m);
  dp[0] = 0;
  for(int i=0; i<m; ++i){
    val |= (int)message[i]<<i;
  }
  for(int i=0; i<b; ++i){
    if(!mem[i]){
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + j] = dp[i*(1<<m) + j];
      }
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])] = min((int)dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])], dp[i*(1<<m)+j]+1);
      }
    } else {
      for(int j=0; j < 1<<m; ++j){
        dp[(i+1)*(1<<m) + (j^bit_vecs[m][i])] = dp[i*(1<<m) + j];
      }
    }
  }
  if(dp[b*(1<<m)+val] >= INF)
    return false;
  int cnt = dp[b*(1<<m)+val];
  for(int i=b-1; i>=0; --i){
    if(!mem[i]){
      if(dp[i*(1<<m) + (val^bit_vecs[m][i])] == cnt-1){
        --cnt;
        val ^= bit_vecs[m][i];
        mem[i] = true;
      }
    } else {
      val ^= bit_vecs[m][i];
    }
  }
  return true;
}

bool encode(vector<bool> &mem, vector<bool> message){
  for(int i=0; i<bucket_cnt; ++i){
    int mem_l = i*b/bucket_cnt;
    int mem_r = (i+1)*b/bucket_cnt;
    int message_l = i*m/bucket_cnt;
    int message_r = (i+1)*m/bucket_cnt;
    vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
    vector<bool> message_seg(message.begin()+message_l, message.begin()+message_r);
    bool suc = small_encode(mem_seg, message_seg);
    if(!suc)
      return false;
    copy(mem_seg.begin(), mem_seg.end(), mem.begin()+mem_l);
  }
  return true;
}

void init_decode(int b, int m){
  init(b, m);
}

vector<bool> small_decode(vector<bool> mem, int m){
  int res = 0;
  int b = mem.size();
  for(int i=0; i<b; ++i){
    if(mem[i]){
      res ^= bit_vecs[m][i];
    }
  }
  vector<bool> out;
  for(int i=0; i<m; ++i){
    out.push_back(res & (1<<i));
  }
  return out;
}
vector<bool> decode(vector<bool> mem){
  vector<bool> message(m);
  for(int i=0; i<bucket_cnt; ++i){
    int mem_l = i*b/bucket_cnt;
    int mem_r = (i+1)*b/bucket_cnt;
    //cerr<<"mem: "<<mem_l<<" "<<mem_r<<endl;
    int message_l = i*m/bucket_cnt;
    int message_r = (i+1)*m/bucket_cnt;
    //cerr<<"message: "<<message_l<<" "<<message_r<<endl;
    vector<bool> mem_seg(mem.begin()+mem_l, mem.begin()+mem_r);
    vector<bool> message_seg = small_decode(mem_seg, message_r-message_l);
    copy(message_seg.begin(), message_seg.end(), message.begin()+message_l);
  }
  return message;
}

int main(){
  srand(1);
  int type;
  cin>>type;
  int b, m;
  cin>>b>>m;
  if(type){
    init_decode(b, m);
    vector<bool> mem(b);
    string mem_str;
    string message_str(m, '0');
    while(true){
      bool cont;
      cin>>cont;
      if(!cont)
        break;
      cin>>mem_str;
      for(int i=0; i<b; ++i)
        mem[i] = mem_str[i]-'0';
      vector<bool> message = decode(mem);
      for(int i=0; i<m; ++i)
        message_str[i] = '0'+message[i];
      cout<<message_str<<endl;
    }
  } else {
    init_encode(b, m);
    vector<bool> mem(b);
    vector<bool> message(m);
    string mem_str;
    string message_str;
    while(true){
      bool cont;
      cin>>cont;
      if(!cont)
        break;
      cin>>mem_str>>message_str;
      for(int i=0; i<b; ++i)
        mem[i] = mem_str[i]-'0';
      for(int i=0; i<m; ++i)
        message[i] = message_str[i]-'0';
      bool suc = encode(mem, message);
      cout<<suc;
      if(suc){
        if(decode(mem) != message){
          cerr<<"Encode error"<<endl;
        }
        for(int i=0; i<b; ++i)
          mem_str[i] = '0'+mem[i];
        cout<<'\n'<<mem_str;
      }
      cout<<endl;
    }
  }
}
