//Rand hash: time expected O(n^2), mem O(n) 
#include <algorithm>
#include <chrono>
#include <ctime>
#include <iostream>
#include <random>
#include <string>
#include <vector>

using namespace std;
typedef long long ll;
const ll mod = 1e9 + 7;
const ll mult = 257;
const int nmax = 5e3 + 5;
ll mult_pow[nmax];

mt19937 gen;
clock_t c_start;

int solve_noflip(string &a, string &b, clock_t c_end, int *loc) {
  string s[2] = {a, b};
  int n[2] = {a.size(), b.size()};
  int l[2];
  int r[2];
  int res = 0;
  while (clock() < c_end) {
    for (int i = 0; i < 2; ++i) {
      l[i] = gen() % n[i];
      r[i] = l[i];
    }
    if (2 * (min(l[0], l[1]) + min(n[0] - r[0], n[1] - r[1])) <= res) continue;
    while (min(l[0], l[1]) > 0 && s[0][l[0] - 1] == s[1][l[1] - 1]) {
      --l[0];
      --l[1];
    }
    if (2 * (r[0] - l[0] + min(n[0] - r[0], n[1] - r[1])) <= res) continue;
    while (r[0] < n[0] && r[1] < n[1] && s[0][r[0]] == s[1][r[1]]) {
      ++r[0];
      ++r[1];
    }
    int len = r[0] - l[0];
    if (2 * len <= res) continue;
    if (len > res) {
      res = len;
      loc[0] = l[0];
      loc[1] = l[1];
    }
    for (int i = 0; i < 2; ++i) {
      ll h[2]{};
      for (int j = 1; j <= min(len, min(n[i] - r[i], l[!i])); ++j) {
        h[i] += s[i][r[i] + j - 1] * mult_pow[j - 1];
        h[i] %= mod;
        h[!i] *= mult;
        h[!i] += s[!i][l[!i] - j];
        h[!i] %= mod;
        if (j > res - len && h[0] == h[1]) {
          res = len + j;
          loc[i] = l[i];
          loc[!i] = l[!i] - j;
        }
      }
    }
  }
  return res;
}

int solve(string &s, string &t, int *loc) {
  int loc_noflip[2];
  int res = solve_noflip(s, t, c_start + CLOCKS_PER_SEC * 45 / 100, loc_noflip);
  reverse(t.begin(), t.end());
  int loc_flip[2];
  int res_flip = solve_noflip(s, t, c_start + CLOCKS_PER_SEC * 97 / 100, loc_flip);
  if (res_flip > res) {
    res = res_flip;
    loc[0] = loc_flip[0];
    loc[1] = (int)t.size() - loc_flip[1] - res;
  } else {
    for (int i = 0; i < 2; ++i) loc[i] = loc_noflip[i];
  }
  return res;
}

int main() {
  c_start = std::clock();

  ll seed = static_cast< ll >(std::chrono::system_clock::now().time_since_epoch().count());
  gen = mt19937(seed);
  gen.discard(gen.state_size);

  mult_pow[0] = 1;
  for (int i = 1; i < nmax; ++i) {
    mult_pow[i] = mult_pow[i - 1] * mult % mod;
  }

  cin.sync_with_stdio(false);
  string s, t;
  cin >> s >> t;
  int loc[2];
  int res = solve(s, t, loc);
  cout << res << '\n';
  if (res) {
    cout << loc[0] << ' ' << loc[1] << '\n';
  }
  return 0;
}
