//DP: time O(n^3), mem O(n^2)

#include <algorithm>
#include <iostream>
#include <string>
#include <vector>

using namespace std;

int solve_noflip(string s, string t, int *loc) {
  int n = s.size();
  int m = t.size();
  int res = 0;
  vector< vector< int > > match_len(n, vector< int >(m));
  vector< vector< int > > forw_back_len(n + 1, vector< int >(m + 1));
  vector< vector< int > > back_forw_len(n + 1, vector< int >(m + 1));
  for (int i = n - 1; i >= 0; --i) {
    for (int j = m - 1; j >= 0; --j) {
      if (s[i] == t[j]) {
        match_len[i][j] = 1;
        if (i + 1 < n && j + 1 < m) match_len[i][j] += match_len[i + 1][j + 1];
      }
      for (int k = 1; k <= match_len[i][j]; ++k) {
        int ni = i + k;
        int nj = j + k;
        forw_back_len[i][nj] = max(forw_back_len[i][nj], k);
        back_forw_len[ni][j] = max(back_forw_len[ni][j], k);
      }
    }
  }
  for (int i = n; i >= 0; --i) {
    for (int j = m; j >= 0; --j) {
      if (forw_back_len[i][j] + back_forw_len[i][j] > res) {
        res = forw_back_len[i][j] + back_forw_len[i][j];
        loc[0] = i - back_forw_len[i][j];
        loc[1] = j - forw_back_len[i][j];
      }
    }
  }
  return res;
}

int solve(string s, string t, int *loc) {
  int loc_noflip[2];
  int res = solve_noflip(s, t, loc_noflip);
  reverse(t.begin(), t.end());
  int loc_flip[2];
  int res_flip = solve_noflip(s, t, loc_flip);
  if (res_flip > res) {
    res = res_flip;
    loc[0] = loc_flip[0];
    loc[1] = (int)t.size() - loc_flip[1] - res;
  } else {
    for (int i = 0; i < 2; ++i) loc[i] = loc_noflip[i];
  }
  return res;
}

int main() {
  string s, t;
  cin >> s >> t;
  int loc[2];
  int res = solve(s, t, loc);
  cout << res << '\n';
  if (res) {
    cout << loc[0] << ' ' << loc[1] << '\n';
  }
  return 0;
}
