//substring combination: time expected O(n^3)

#include <algorithm>
#include <chrono>
#include <ctime>
#include <iostream>
#include <random>
#include <set>
#include <string>
#include <tuple>
#include <vector>

using namespace std;
typedef long long ll;

mt19937_64 gen;

int solve_noflip(string a, string b, int *loc) {
  string s[2] = {a, b};
  int n[2] = {a.size(), b.size()};
  int l[2];
  int r[2];
  set< tuple< int, int, int > > segs;  // l_a, l_b, len
  clock_t c_start = clock();
  clock_t c_end = c_start + CLOCKS_PER_SEC * 12 / 100;
  while (segs.size() < 1e5 && clock() < c_end) {
    for (int i = 0; i < 2; ++i) {
      l[i] = gen() % n[i];
      r[i] = l[i];
    }
    while (min(l[0], l[1]) > 0 && s[0][l[0] - 1] == s[1][l[1] - 1]) {
      --l[0];
      --l[1];
    }
    while (r[0] < n[0] && r[1] < n[1] && s[0][r[0]] == s[1][r[1]]) {
      ++r[0];
      ++r[1];
    }
    int len = r[0] - l[0];
    segs.emplace(l[0], l[1], len);
  }
  cerr << "size = " << segs.size() << '\n';
  vector< tuple< int, int, int > > segs_vec(segs.begin(), segs.end());  // l_a, l_b, len
  int res = 0;
  c_end = c_start + CLOCKS_PER_SEC * 25 / 100;
  while (clock() < c_end) {
    int i = gen() % segs_vec.size();
    int l_a1;
    int l_b1;
    int len1;
    tie(l_a1, l_b1, len1) = segs_vec[i];
    if (len1 > res) {
      res = len1;
      loc[0] = l_a1;
      loc[1] = l_b1;
    }

    for (int j = i + 1; j < (int)segs_vec.size(); ++j) {
      int l_a2;
      int l_b2;
      int len2;
      tie(l_a2, l_b2, len2) = segs_vec[j];
      if (l_a2 > l_a1 + len1) break;
      if (l_b1 > l_b2 && l_b1 <= l_b2 + len2) {
        int cand = l_a2 - l_a1 + len2 - (l_b2 + len2 - l_b1);
        if (cand > res) {
          res = cand;
          loc[0] = l_a1;
          loc[1] = l_b2;
        }
      }
    }
  }
  return res;
}

int solve(string s, string t, int *loc) {
  int loc_noflip[2];
  int res = solve_noflip(s, t, loc_noflip);
  reverse(t.begin(), t.end());
  int loc_flip[2];
  int res_flip = solve_noflip(s, t, loc_flip);
  if (res_flip > res) {
    res = res_flip;
    loc[0] = loc_flip[0];
    loc[1] = (int)t.size() - loc_flip[1] - res;
  } else {
    for (int i = 0; i < 2; ++i) loc[i] = loc_noflip[i];
  }
  return res;
}

int main() {
  ll seed = static_cast< ll >(std::chrono::system_clock::now().time_since_epoch().count());
  gen = mt19937_64(seed);
  gen.discard(gen.state_size);

  string s, t;
  cin >> s >> t;
  int loc[2];
  int res = solve(s, t, loc);
  cout << res << '\n';
  if (res) {
    cout << loc[0] << ' ' << loc[1] << '\n';
  }
  return 0;
}
